#!/usr/bin/env python
from execute.command_function import get_command_execute
from gatheros import __name__ as module_name
from execute.execution_unit import ExecutionUnit

import argparse
import sys, os
import json

base_dir = os.sep.join(__file__.split( os.sep )[:-1])

parser = argparse.ArgumentParser( description = "A tool for remote system enumeration" )

parser.add_argument( "--command-file", '-c',\
					help = "The file that contains the commands to run on the remote system in JSON format",\
					default = base_dir+"/commands/LinuxEnum.json" )

subparsers = parser.add_subparsers( help = "The connection type with the remote host", dest='command')

localhost_parser = subparsers.add_parser( "local" )

ssh_parser = subparsers.add_parser("ssh")
ssh_parser.add_argument("connection", default = "root@172.0.0.1",\
						 help = "SSH connection string. example: 'user@address'")
ssh_parser.add_argument("--port", '-p', help = "TCP port to SSH (default: 22)", default = 22, type = int)
ssh_parser.add_argument("--password", '-P',\
						help = "Password to use when connecting to server. If password is not given it's asked from the tty.")


bind_parser = subparsers.add_parser("bind")
bind_parser.add_argument("IP", help = "The IP address of the remote host")
bind_parser.add_argument("--port", '-p', help = "TCP port to connect (default: 4444)", default = 4444, type = int)


reverse_parser = subparsers.add_parser("reverse")
reverse_parser.add_argument("--port", '-p', help = "TCP port to wait for the shell (default: 4444)", default = 4444, type = int)

parser.add_argument("--output-file", '-o',\
					help = "The file to save the command output" )

# parser.add_argument("--quiet", '-q', help = "")

def command_loader( json_file ) :
	with open( json_file, 'r' ) as file :
		ret = json.load( file )
	return ret


def main( arguments = sys.argv[1:] ) :

	args = parser.parse_args( arguments )
	command_dict = command_loader( args.command_file )

	execute_command = get_command_execute( args )
	execUnit = ExecutionUnit( execute_command, command_dict )
	command_dict = execUnit.execute()

	json_dump = json.dumps( command_dict, indent = 1 )
	
	if args.output_file :
		with open( args.output_file, 'w' ) as toWrite :
			toWrite.write( json_dump )
	else :
		print json_dump

	# return command_dict


if __name__ == '__main__' :

	main()



import socket
import paramiko
import os, sys
import getpass

client = None
ssh = None

def runSocketCommand( comm ) :
	client.send( ' ' + comm + '\n')
	return client.recv(4096*2)

def runLocalhostCommand( comm ) :
	return os.popen( " " + comm ).read()

def runSSHCommand( comm ) :
	stdin, stdout, stderr = ssh.exec_command( comm )
	out = stdout.read()
	if not out :
		return stderr.read()
	return out


def get_command_execute ( args ) :
	global client
	global ssh
	if args.command == "bind" :
		client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		address = (args.IP, args.port )
		client.connect( address )
		runCommand = runSocketCommand


	elif args.command == "reverse" :
		server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		server.setsockopt( socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		server.bind( ("0.0.0.0", args.port ) )
		print "Waiting for the Reverse Shell at port %d" % args.port
		try :
			server.listen(5)
		except KeyboardInterrupt :
			print "Aborted by user..."
			sys.exit(-2)
		client, address = server.accept()
		runCommand = runSocketCommand


	elif args.command == "local" :
		runCommand = runLocalhostCommand


	elif args.command == "ssh" :

		user, host = args.connection.split('@')[:2]
		password = args.password
		if not password :
			password = getpass.getpass("(SSH) Password for user '%s': " % user)

		ssh = paramiko.SSHClient()
		ssh.set_missing_host_key_policy( paramiko.AutoAddPolicy() )
		try :
			ssh.connect( host , username = user, password = password, port = args.port )
		except paramiko.ssh_exception.AuthenticationException :
			print "Authentication Failed"
			sys.exit(-1)
		runCommand = runSSHCommand

	return runCommand


import datetime
import os
import re


def createCodename( name ) :
 	codename = name.lower()
	codename = re.sub( "\s", "", codename )
	codename = re.sub( "\W", "", codename )
	return codename

def createEmptyStruct () :
	ret = {}
	ret['CommandGroups'] = {}
	ret['Commands'] = {}
	ret['OperatingSystem'] = ""
	ret['Populated'] = False
	ret['DependenyTokens'] = []
	ret['Metadata'] = {
		'author' : '',
		'description' : '',
		'creation_date' : datetime.datetime.now().strftime ( "%Y-%m-%d" ),
		'reference': ''
	}
	return ret


def createEmptyCommandGroup ( name ) :
	codename = createCodename( name )
	ret = {}
	ret['name'] = name
	ret['index'] = createEmptyCommandGroup.index
	ret['Commands'] = []
	createEmptyCommandGroup.index += 1
	return ( codename, ret )
createEmptyCommandGroup.index = 0


def createEmptyCommand ( zero_index = False ) :
	if zero_index :
		createEmptyCommand.index = 0

	unique = os.urandom(12).encode( 'hex' )
	ret = {}
	ret['command'] = ''
	ret['description'] = ''
	ret['index'] = createEmptyCommand.index
	ret['depends'] = []
	ret['unlocks'] = []
	ret['response_filter'] = ''	# the 'response' variable will contain the command response
	ret['show'] = True
	createEmptyCommand.index += 1

	return ( unique, ret )
createEmptyCommand.index = 0



def addCommand( struct, comm_group_codename, comm_id, command ) :
	struct['Commands'][comm_id] = command
	struct['CommandGroups'][comm_group_codename]['Commands'][comm_group_codename].append(comm_id)


def deleteCommand( comm_id, comm_group_codename = None ) :
	del struct['Commands'][comm_id]
	if comm_group_codename :
		struct['CommandGroups'][comm_group_codename]
	else :
		for commandgroup in struct['CommandGroups'].values() :
			try :
				commandgroup['Commands'].remove( comm_id )
			except :
				pass



#!/usr/bin/env python
from gatheros import __name__ as module_name
from editor.cmd_interface import EditorShell

import argparse
import sys, os, os.path
import json
import cmd

parser = argparse.ArgumentParser( description = "A tool to read/write 'gatheros' command execution files")
parser.add_argument( "file", help = "The gatheros command execution file to open" )
write_argument = '--write'
parser.add_argument( write_argument, '-w', help = "Open the file in 'write mode'",\
					action = 'store_true', default = False )

def main( arguments = sys.argv[1:] ) :

	args = parser.parse_args( arguments )

	if not os.path.isfile(args.file) and not args.write :
		print "Can't create a file without '%s' argument" % write_argument
		sys.exit(-1)


	jsonFile = open( args.file, 'r+w' )

	commandStruct = None
	# commandStruct = json.load( jsonFile )

	# print str(commandStruct)[:100]

	print jsonFile


	shell = EditorShell ( jsonFile )
	shell.cmdloop()


if __name__ == '__main__' :

	main()





from pprint import pprint
import sys



class ExecutionUnit :

	def __init__ ( self, execute_command, commStruct ) :
		self.execute_command = execute_command
		self.commStruct = commStruct
		self.dependencies_met = set()
		self.allCommands = set( self.commStruct['Commands'].keys() ) 
		self.allCommandsDict = self.commStruct['Commands']
		
		self.notExecuted = self.allCommands


	def executeCommandSet( self, commSet ) :

		commFailed = set()

		for comm_id in commSet :
			command = self.commStruct['Commands'][comm_id]
			self.notExecuted.remove( comm_id )

			try :
				response = self.execute_command ( command['command'] ).decode( 'utf8' )
				self.dependencies_met.update( command['unlocks'] )
			except :
				commFailed.add( comm_id )
				print "[!] Command '%s' couldn't be executed." % comm_id
				print "'%s'" % command['command']
				# command['error'] = True
				print sys.exc_info()[0]
				continue

			try :
				filter_ = command['response_filter']
				if filter_ :
					response = self.filterResponse( response, filter_ )
			except KeyError:
				pass
			except :
				print "Command '%s' filter:\n>>> %s\n Couldn't be executed.\n" % (comm_id, filter_)
			command['response'] = response


		# commSet.difference_update( commFailed )
		return commSet


	def filterResponse( self, response, filter_ ) :
		ret = str( eval ( filter_,  ) )
		return ret


	def getReadyCommands( self ) :
		ret = set()
		for comm_id in self.notExecuted :
			command = self.allCommandsDict[ comm_id ]
			commDependencies = set( command['depends'] )
			if commDependencies <= self.dependencies_met :
				ret.add( comm_id )
		return ret


	def execute( self ) :
		wDependencies = set( [ id_ for id_, command in self.allCommandsDict.iteritems()\
										if command['depends'] ] )
		woutDependencies = self.allCommands - wDependencies

		readyCommands = self.getReadyCommands()

		print "Remaining Commands: %d" % len(self.notExecuted)
		while readyCommands :
			self.executeCommandSet( readyCommands )
			print "Remaining Commands: %d" % len(self.notExecuted)
			readyCommands = self.getReadyCommands()

		self.commStruct['Populated'] = True
		return self.commStruct


#!/usr/bin/env python

import show.application as flask

import argparse
import sys, os
import json


parser = argparse.ArgumentParser( description = "A tool for presenting System information gathered by 'gatheros execute' module" )

parser.add_argument("-a" "--address",\
					help = "IP were the %s WebPage will be served" % sys.argv[0], default = 'localhost')
parser.add_argument("-p", "--port",\
					help = "Port were the %s WebPage will be served" % sys.argv[0], type = int, default = 8085)
parser.add_argument("--quiet","-q",\
					help = "Start the service without spawning a browser instance", action = "store_true", default = False)
parser.add_argument("file",\
					help = "Location of the 'gatheros' system info file to open")



def show( command_dict, ip, port ) :
	flask.commStruct = command_dict
	flask.flask_app.run( host = ip, port = port )


def main( arguments = sys.argv[1:] ) :

	args = parser.parse_args( arguments )
	# print args
	with open( args.file, 'r' ) as file :
		command_dict = json.load( file )

	if not args.quiet :
		os.system(" firefox http://localhost:%d &" % args.port)
	else :
		print "Starting Web Application at 'http://localhost:%d'" % args.port

	show( command_dict, args.a__address, args.port )


if __name__ == '__main__' :

	main()


from flask import Flask, request, render_template, abort
import json
import os

base_dir = os.sep.join(__file__.split( os.sep )[:-1])

template_folder = base_dir + os.sep + "templates"
static_folder = base_dir + os.sep + "static"

# print template_folder, static_folder
# print __name__
flask_app_name = __name__.split('.')[0]
# flask_app_name = "gatheros." + __name__.split('.')[0]
# flask_app_name = "gatheros"
# flask_app_name = __name__
# print flask_app_name
flask_app = Flask( flask_app_name,\
					template_folder = template_folder,\
					static_folder = static_folder)

commStruct = {}


@flask_app.route('/')
def index_page() :
	groups = commStruct['CommandGroups']
	tagged = [ (k,v) for k,v in commStruct['Commands'].iteritems() if 'tag' in v]
	group_items = groups.items()
	ordered_groups = sorted(groups.items(), key = lambda i:i[1]['index'] )
	return render_template("index.html",\
		commandGroups = ordered_groups, tagged = tagged,\
		os = commStruct['OperatingSystem'].lower(), populated = commStruct['Populated'], metadata = commStruct['Metadata'].iteritems() )


@flask_app.route('/command/<name>')
def commandsPage( name ) :
	if name not in commStruct['CommandGroups'].keys() :
		return abort(404)

	group = commStruct['CommandGroups'][ name ]
	command_ids = set( group['Commands'] )
	print command_ids
	command_dict = [ (id_, comm) for id_, comm in commStruct['Commands'].iteritems() if id_ in command_ids]
	ordered_commands = sorted(command_dict, key = lambda i:i[1]['index'] )
	# print ordered_commandss
	return render_template("commands.html", title = group['name'], commList = ordered_commands )


@flask_app.route('/search/', methods = ['POST'])
def searchPage() :
	if request.method != 'POST' :
		return index()
	data = request.form
	if 'keyword' not in data :
		return index()
	keyword = data['keyword']

	ret = {}
	ret['Commands'] = []
	for key, command in commStruct['Commands'].iteritems() :
		toSearch = [key]
		toSearch.extend(command.values())
		toSearch = [ v for v in toSearch if isinstance(v, basestring) and v]
		toSearch = ' '.join( toSearch )
		if keyword in toSearch :
			ret['Commands'].append( (key, command) )
	ret['name'] = 'Search for keyword "%s" - %d results' % (keyword, len( ret['Commands'] ))
	return render_template("commands.html", commList = ret['Commands'], title = ret['name'])



#!/usr/bin/env python
import gatheros_exec
import gatheros_show
import gatheros

import argparse
import sys, os
import tempfile

parser = argparse.ArgumentParser( description = "A tool for System Enumeration and Presentation" )

temp = tempfile.NamedTemporaryFile( delete = True )
# print temp
# sys.exit()
parser.add_argument( "--execute", help = "", type = str, default = "-o %s local" % temp.name )
# parser.add_argument( "--file", )

parser.add_argument( "--show", help = "", default = "%s -a localhost" % temp.name )




def main( arguments = sys.argv[1:] ) :

	# print arguments
	args = parser.parse_args( arguments )
	# print args
	print "Running 'execute' module with '%s' arguments" % args.execute
	command_dict = gatheros_exec.main( args.execute.split() )

	print "Running 'show' module with '%s' arguments" % args.show
	gatheros_show.main( args.show.split() )	


if __name__ == '__main__' :

	main()




def populateDict( dict_, recursive = True ) :
	for key in dict_.keys() :
		print "===================="
		print "Editing [%s]:" % key

		if type( dict_[key] ) == dict and recursive:
			value = populateDict( dict_[key] )
		if type( dict_[key] ) == list and recursive:
			value = populateList( dict_[key] )

		else :
			cur_value = dict_[key]
			added_comment = ''
			if cur_value != None :
				added_comment = "[%s]" % cur_value
			value = raw_input( "* Type a value %s: " % ( added_comment ) ).strip()
			if not value :
				# print " - Space found. Continuing"
				# print 
				# print 
				continue

		dict_[key] = value

	return dict_


def populateList( list_ ) :
	eof = '-EOF-'
	print "Type Empty String to submit the list"
	value = raw_input("*	Append next element: ")
	while value != '' :
		list_.append( value )
		value = raw_input( "*	Append next element: ").strip()

	return list_



import cmd
import json
import termcolor

from struct_manager import createEmptyCommandGroup, createEmptyStruct, createEmptyCommand
from populator import populateDict

class EditorShell ( cmd.Cmd ) :


	def __init__ ( self, file ) :
		cmd.Cmd.__init__(self)
		self.file = file
		# self.struct = json.load ( self.file )
		try :
			self.struct = json.load ( self.file )
		except :
			print "[!] Can't open the JSON file, creating a new struct"
			self.struct = createEmptyStruct()

		self.cur_node = self.struct


	def do_add( self, line ) :
		if not line :
			return
		line = line.strip()
		toks = line.split()
		ident = toks[0].lower()
		if 'command' == ident :
			self.do_add_command( ' '.join( toks [1:] ) )
			pass	# add command

		elif 'group' == ident :
			self.do_add_group( ' '.join( toks [1:] ) )

			pass	# add command

		elif 'dependency' == ident :
			pass	# add command

		else :
			print " '%s' not available subcommand!" % ident


	def do_add_group( self, line ) :
		if not line :
			print "Need a 'name' "
			return
		line = line.strip()
		toks = line.split()
		codename, group = createEmptyCommandGroup( toks[0] )
		populateDict(group)
		print group
		self.struct['CommandGroups'][ codename ] = group


	def do_add_command( self, line ) :
		if not line :
			print "Need a 'group code name' "
			return
		line = line.strip()
		toks = line.split()
		codename = toks[0]
		unique, command = createEmptyCommand( )
		populateDict( command )
		self.struct['CommandGroups'][ codename ]['Commands'][ unique ] = command
		print "Command '%s' created!" % unique


	def do_add_dependency( self, line ) :
		pass




	def do_show_command( self, line ) :

		pass


	def do_edit_command( self, line ) :
		if not line :
			print "Need a 'command identifier' "
			return
		line = line.strip()
		toks = line.split()
		ident = toks[0]
		for gname, group in self.struct['CommandGroups'].iteritems() :
			try :
				comm = group['Commands'][ident]
				break
			except :
				pass

		if not comm :
			print "Identifier '%s' doesn't exist" % comm
			return
		populateDict( comm, False )


	def do_edit_group( self, line ) :
		if not line :
			print "Need a 'command identifier' "
			return
		line = line.strip()
		toks = line.split()
		gname = toks[0]
		group = self.struct['CommandGroups'][gname]
		populateDict( group )

		if not comm :
			print "Identifier '%s' doesn't exist" % comm
			return


	def do_list( self, line ) :
		for group in self.struct['CommandGroups'].keys() :
			print group

	def do_list_commands( self, line ) :
		for gname, group in self.struct['CommandGroups'].iteritems() :
			print "=========== %s ===========" % group['name']
			for k, v in group['Commands'].iteritems() :
				print '''{0:24} -| {1:<64}\n-> {2:<64}
'''.format( k, v['command'].encode('utf8'), v['description'].encode('utf8') )
			print "=========== --- ===========" 
			print 


	def do_save( self, line ) :

		self.file.seek(0)
		self.file.write( json.dumps( self.struct, indent = 1 ) + '\n' )
		self.file.truncate()


	def do_create_dependency( self, line ) :
		if not line :
			print "Need a 'name' "
			return
		line = line.strip()
		toks = line.split()
		dep = toks[0]
		self.struct['DependencyTokens'].append( dep )


	def do_d_list( self, line ) :
		for group in self.struct.keys() :
			print group


	def do_d_show( self, line ) :
		print json.dumps( self.struct, indent = 1 )

